<?php 
/* 
 * 	   Copyright 2010 WALTHER Jrmy & MOLE Bruno
 *    Zoku Server
 *    Version 1.0-beta
 *    
 *    Contact the authors :
 *    WALTHER Jrmy	: jeremy.walther@ftad.fr
 *    MOLE Bruno		: bruno.mole@ftad.fr
 *    
 *    This file is part of Zoku Server.
 *
 *    Zoku Server is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Zoku Server is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Zoku Server.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
?>
<?php

/**
 * Indicates if user is logged.
 *
 * @var bool
 * @access private
 */
$zs_user_logged = false;

/**
 * Contains user's informations.
 *
 * @var ZS_User
 * @access private
 */
$zs_user_infos = new ZS_User();
$zs_user_infos->admin = zs_adminIntToArray($zs_user_infos->admin);

/**
* Format a result to be returned to client as an answer.
*
* @param	mixed	$val	The value to format
* @param	bool	$force_conversion	Force conversion type
* @return	string	The formatted result
* @access	private
*/
function formatResult($result = "", $force_conversion = false)
{
	if (is_bool($result))
	{
		if ($result)
		{
			return "TRUE";
		}
		else
		{
			return "FALSE";
		}
	}
	
	if (is_array($result) && $force_conversion)
	{
		//return zs_serialize_array($result);
		/*foreach ($result as $key => $value)
		{
			$result[$key] = formatResult($value);
		}*/
		return new soapval('row', 'Row', $result, false, 'urn:ZokuRow');
	}
	
	if (empty($result) || $result == "")
	{
		return "VOID";
	}
	
	return $result;
}

/**
* Format an error message to be returned to client.
*
* @param	string	$error	Human readable error message.
* @param	mixed	$details	Details of the error. Can be a string or an array of strings.
* @param	string	$type	Type of the error, can be SERVER or CLIENT.
* @return	SOAP_Fault	SOAP Fault.
* @access	private
*/
function formatError($error = "ERROR", $details = "", $type = "SERVER")
{
	return new nusoap_fault($type, "CLIENT", $error, $details);
}

/**
* Format an input to be used by the server.
* Especially used to echap sql injection and other beauties.
*
* @param	mixed	$val	The value to format
* @return	string	The formatted result
* @access	private
*/
function formatInput($input)
{
	if (is_array($input))
	{
		$result = array();
		foreach ($input as $key => $value)
		{
			$result[formatInput($key)] = formatInput($value);
		}
		return $result;
	}
	
	$input = zs_noSQL($input);
	$input = zs_Echap($input);
	
	return $input;
}

/**
* Return true if a user is logged on the webservice.
*
* @return	bool	connection status
* @access	private
*/
function isUserLogged()
{
	global $zs_user_logged;
	return $zs_user_logged;
}

/**
* Get user's (if one is logged) informations.
* If no user is logged, this will return False.
*
* @return	ZS_User	User's informations
* @access	private
*/
function getUserInfos()
{
	global $zs_user_logged;
	if ($zs_user_logged)
	{
		global $zs_user_infos;
		return $zs_user_infos;
	}
	return false;
}

/**
* Return true if mail sending is enabled.
*
* @return	bool	mail using status
* @access	private
*/
function isMailEnabled()
{
	global $zs_enable_mail;
	return $zs_enable_mail;
}

/**
* Establish a session, and logon an user.
*
* @param	string	$username	User's name
* @param	string	$password	User's password (md5 hash)
* @return	bool	True if succeeded, False otherwise
* @access	private
*/
function headerAuthentify($username, $password)
{
	$session = new ZS_Session();
	$username = formatInput($username);
	$password = formatInput($password);
	if ($session->Connect($username, $password))
	{
		global $zs_user_logged;
		global $zs_user_infos;
		$zs_user_logged = $session->isLogged();
		$zs_user_infos = $session->getUser();
		$zs_user_infos->admin = zs_adminIntToArray($zs_user_infos->admin);
		$zs_user_infos->updateLog();
		return true;
	}
	return false;
}

/**
* Show login debug informations.
*
* @return	array	(Username, Password, isLogged)
* @access	public
*/
function debugLogin()
{
	global $zs_user_username;
	global $zs_user_password;
	global $zs_user_logged;
	return array(	'username'	=>	formatResult($zs_user_username),
					'password'	=> 	formatResult($zs_user_password),
					'logged'	=> 	formatResult($zs_user_logged));
}

/**
* Show SOAP data (http raw post data).
*
* @return	string	Content of php://input
* @access	public
*/
function debugHeaders()
{
	global $zs_server_postdata;
	return $zs_server_postdata;
}

/**
* Create an user into the database.
*
* @deprecated
* @param	array	$row	User data
* @return	string	ID of created user, or FALSE
* @access	public
*/
/*function createUser($row)
{
	//$row = formatInput(zs_unserialize_array($row));
	$row = formatInput($row);
	$user = new ZS_User($row);
	if ($user->checkConsistency())
	{
		if ($result = $user->createInDB())
		{
			return formatResult($result);
		}
		return formatError("Cannot create user in database.");
	}
	return formatError("All required vars must be set.");
}*/

/**
* Create a card into the database.
*
* @param	array	$row	Card data
* @return	string	ID of created card, or FALSE
* @access	public
*/
function createCard($row)
{
	//$row = formatInput(zs_unserialize_array($row));
	$row = formatInput($row);
	$card = new ZS_Card($row);
	if ($card->checkConsistency())
	{
		if ($result = $card->createInDB())
		{
			if (isMailEnabled())
			{	
				$mail = new PHPMailer(false);
				$mail->IsSendmail(); // telling the class to use SendMail transport
				$mail->AddReplyTo($card->email, $card->first_name." ".zs_strToUpper($card->last_name));
				$mail->AddAddress($card->email, $card->first_name." ".zs_strToUpper($card->last_name));
				$mail->SetFrom($card->email, $card->first_name." ".zs_strToUpper($card->last_name));
				$mail->Subject = "Zoku : Cration de votre carte";
				//$mail->AltBody = 'To view the message, please use an HTML compatible email viewer!'; // optional - MsgHTML will create an alternate automatically
				$html = "<html><body><p>Bonjour,</p><p>Votre carte a t cre, voici les informations qui lui sont lies :</p>";
				$html .= "<ul>";
				foreach ($card->getVars() as $key => $value)
				{
					$html .= "<li>$key = $value</li>";
				}
				$html .= "</ul><p>Vous pouvez modifier ensuite ces informations en vous connectant  l'interface web Zoku.</p></body></html>";
				$mail->MsgHTML($html);
				//$mail->AddAttachment('images/phpmailer.gif');      // attachment
				//$mail->AddAttachment('images/phpmailer_mini.gif'); // attachment
				$mail->Send();
			}
			return formatResult($card->id);
		}
		return formatError("Cannot create card in database.");
	}
	return formatError("All required vars must be set.");
}

/**
* Create a group into the database.
* Note : a session must be set in order to use this.
*
* @param	array	$row	Group data
* @return	string	ID of created group, or FALSE
* @access	public
*/
function createGroup($row)
{
	if (isUserLogged())
	{
		//$row = formatInput(zs_unserialize_array($row));
		$row = formatInput($row);
		$group = new ZS_Group($row);
		if ($group->checkConsistency())
		{
			$session = getUserInfos();
			if ($session->getGroupId() == 0)
			{
				if ($result = $group->createInDB())
				{
					$newRights = array(	"group"			=> true,
										"card"			=> true,
										"event"			=> true,
										"eventmember"	=> true);
					$session->setAdmin($newRights);
					$session->setGroupId($group->id);
					return formatResult($group->id);
				}
				return formatError("Cannot create group in database.");
			}
			return formatError("You already have a group.");
		}
		return formatError("All required vars must be set.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Create a super group into the database.
* Note : a session must be set in order to use this.
*
* @param	array	$row	SuperGroup data
* @return	string	ID of created super group, or FALSE
* @access	public
*/
function createSuperGroup($row)
{
	if (isUserLogged())
	{
		$session = getUserInfos();
		if ($session->admin["super"] || $session->admin["admin"] && $session->admin["group"])
		{
			//$row = formatInput(zs_unserialize_array($row));
			$row = formatInput($row);
			$supergroup = new ZS_SuperGroup($row);
			if ($supergroup->checkConsistency())
			{
				if ($result = $supergroup->createInDB())
				{
					return formatResult($result);
				}
				return formatError("Cannot create super group in database.");
			}
			return formatError("All required vars must be set.");
		}
		return formatError("You are not allowed to do this.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Create an event into the database.
* Note : a session must be set in order to use this.
*
* @param	array	$row	Event data
* @return	string	ID of created event, or FALSE
* @access	public
*/
function createEvent($row)
{
	if (isUserLogged())
	{
		//$row = formatInput(zs_unserialize_array($row));
		$row = formatInput($row);
		$event = new ZS_Event($row);
		if ($event->checkConsistency())
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["event"])
			{
				if ($result = $event->createInDB())
				{
					return formatResult($result);
				}
				return formatError("Cannot create event in database.");
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("All required vars must be set.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Create an event member into the database.
* Note : a session must be set in order to use this.
*
* @param	array	$row	Event Member data
* @return	string	ID of created event member, or FALSE
* @access	public
*/
function createEventMember($row)
{
	if (isUserLogged())
	{
		//$row = formatInput(zs_unserialize_array($row));
		$row = formatInput($row);
		$eventmember = new ZS_EventMember($row);
		if ($eventmember->checkConsistency())
		{
			$event = new ZS_Event();
			if ($event->selectById($eventmember->event_id))
			{
				$session = getUserInfos();
				if ($session->admin["super"] || $session["admin"] && $session->admin["eventmember"] || $session->admin["eventmember"] && $session->getGroupId() == $event->group_id)
				{
					$eventmember->controller_id = $session->id;
					if ($result = $eventmember->createInDB())
					{
						return formatResult($result);
					}
					return formatError("Cannot create event member in database.");
				}
				return formatError("You are not allowed to do this.");
			}
			return formatError("Cannot select this event.");
		}
		return formatError("All required vars must be set.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get the url of the specified card.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Card ID
* @param	string	$key Card Key
* @return	string	URL of the card
* @access	public
*/
function showCard($id, $key)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$key = formatInput($key);
		$card = new ZS_Card();
		if ($card->selectByIdAndKey($id, $key))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"] || $session->admin["card"] && $session->getGroupId() == $card->group_id || $session->id == $card->id)
			{
				return formatResult($card->generateCard());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this card.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get members of a group.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Group ID
* @return	array	Array of Cards data
* @access	public
*/
function getGroupMembers($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$group = new ZS_Group();
		if ($group->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["group"] || $session->admin["group"] && $session->getGroupId() == $group->id)
			{
				return formatResult($group->getMembers());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this group.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get members of an event.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Event ID
* @return	array	Array of EventMembers data
* @access	public
*/
function getEventMembers($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$event = new ZS_Event();
		if ($event->selectById($id))
		{
			return formatResult($event->getMembers());
		}
		return formatError("Cannot select this event.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get groups of a super group.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Group ID
* @return	array	Array of Cards data
* @access	public
*/
function getSuperGroupGroups($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$supergroup = new ZS_SuperGroup();
		if ($supergroup->selectById($id))
		{
			return formatResult($supergroup->getGroups());
		}
		return formatError("Cannot select this super group.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get required var names to create a Card.
*
* @return	array	Required var names to create a card
* @access	public
*/
function getCardRequiredVars()
{
	$card = new ZS_Card();
	return formatResult($card->getRequiredVarNames());
}

/**
* Get required var names to create a Group.
*
* @return	array	Required var names to create a group
* @access	public
*/
function getGroupRequiredVars()
{
	$group = new ZS_Group();
	return formatResult($group->getRequiredVarNames());
}

/**
* Get required var names to create an Event.
*
* @return	array	Required var names to create an event
* @access	public
*/
function getEventRequiredVars()
{
	$event = new ZS_Event();
	return formatResult($event->getRequiredVarNames());
}

/**
* Get required var names to create an EventMember.
*
* @return	array	Required var names to create an event member
* @access	public
*/
function getEventMemberRequiredVars()
{
	$eventmember = new ZS_EventMember();
	return formatResult($eventmember->getRequiredVarNames());
}

/**
* Get public vars of a Card.
*
* @param	int	$id	Card ID
* @param	string	$key	Card Key
* @return	array	Card public vars
* @access	public
*/
function getCardPublicVars($id)
{
	$id = formatInput($id);
	$card = new ZS_Card();
	if ($card->selectById($id))
	{
		return formatResult($card->getPublicVars());
	}
	return formatError("Cannot select this card.");
}

/**
* Get public vars of an Event.
*
* @param	int	$id	Event ID
* @return	array	Event public vars
* @access	public
*/
function getEventPublicVars($id)
{
	$id = formatInput($id);
	$event = new ZS_Event();
	if ($event->selectById())
	{
		return formatResult($event->getPublicVars());
	}
	return formatError("Cannot select this event.");
}

/**
* Get public vars of an EventMember.
*
* @param	int	$id	EventMember ID
* @return	array	EventMember public vars
* @access	public
*/
function getEventMemberPublicVars($id)
{
	$id = formatInput($id);
	$eventmember = new ZS_EventMember();
	if ($eventmember->selectById())
	{
		return formatResult($eventmember->getPublicVars());
	}
	return formatError("Cannot select this event member.");
}

/**
* Get public vars of a Group.
*
* @param	int	$id	Group ID
* @return	array	Group public vars
* @access	public
*/
function getGroupPublicVars($id)
{
	$id = formatInput($id);
	$group = new ZS_Group();
	if ($group->selectById($id))
	{
		return formatResult($group->getPublicVars());
	}
	return formatError("Cannot select this group.");
}

/**
* Get private vars of a Card.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Card ID
* @return	array	Card private vars
* @access	public
*/
function getCardPrivateVars($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$card = new ZS_Card();
		if ($card->selectById($id))
		{
			return formatResult($card->getPrivateVars());
		}
		return formatError("Cannot select this card.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get private vars of an Event.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Event ID
* @return	array	Event private vars
* @access	public
*/
function getEventPrivateVars($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$event = new ZS_Event();
		if ($event->selectById())
		{
			return formatResult($event->getPrivateVars());
		}
		return formatError("Cannot select this event.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get private vars of an EventMember.
* Note : a session must be set in order to use this.
*
* @param	int	$id	EventMember ID
* @return	array	EventMember private vars
* @access	public
*/
function getEventMemberPrivateVars($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$eventmember = new ZS_EventMember();
		if ($eventmember->selectById())
		{
			return formatResult($eventmember->getPrivateVars());
		}
		return formatError("Cannot select this event member.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get private vars of a Group.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Group ID
* @return	array	Group private vars
* @access	public
*/
function getGroupPrivateVars($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$group = new ZS_Group();
		if ($group->selectById())
		{
			return formatResult($group->getPrivateVars());
		}
		return formatError("Cannot select this group.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get all vars of a Card.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Card ID
* @return	array	All card vars
* @access	public
*/
function getCardAllVars($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$card = new ZS_Card();
		if ($card->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"] || $session->id == $id)
			{
				return formatResult($card->getVars());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this card.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get all vars of an Event.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Event ID
* @return	array	All event vars
* @access	public
*/
function getEventAllVars($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$event = new ZS_Event();
		if ($event->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["event"] || $session->admin["event"] && $session->getGroupId() == $event->group_id)
			{
				return formatResult($event->getVars());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this event.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get all vars of an EventMember.
* Note : a session must be set in order to use this.
*
* @param	int	$id	EventMember ID
* @return	array	All event member vars
* @access	public
*/
function getEventMemberAllVars($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$eventmember = new ZS_EventMember();
		if ($eventmember->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["eventmember"] || $session->admin["eventmember"] && $session->getGroupId() == $eventmember->getEventGroupId())
			{
				return formatResult($eventmember->getVars());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this event member.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get all vars of a Group.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Group ID
* @return	array	All group vars
* @access	public
*/
function getGroupAllVars($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$group = new ZS_Group();
		if ($group->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["group"] || $session->id == $group->owner_id || $session->admin["group"] && $session->getGroupId() == $group->id)
			{
				return formatResult($group->getVars());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this group.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Update a Card into the database with the specified data.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Card ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function updateCard($row)
{
	if (isUserLogged())
	{
		//$row = formatInput(zs_unserialize_array($row));
		$row = formatInput($row);
		$card = new ZS_Card();
		if ($card->selectById($row['id']))
		{
			$card->createFromArray($row);
			if (isset($row['password']))
			{
				$card->hashPassword();
			}
			if ($card->checkConsistency())
			{
				$session = getUserInfos();
				if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"] || $session->id = $card->id)
				{
					return formatResult($card->updateIntoDB());
				}
				return formatError("You are not allowed to do this.");
			}
			return formatError("All required vars must be set.");
		}
		return formatError("Cannot select this card.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Update a Group into the database with the specified data.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Group ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function updateGroup($row)
{
	if (isUserLogged())
	{
		//$row = formatInput(zs_unserialize_array($row));
		$row = formatInput($row);
		$group = new ZS_Group();
		if ($group->selectById($row['id']))
		{
			$group->createFromArray($row);
			if ($group->checkConsistency())
			{
				$session = getUserInfos();
				if ($session->admin["super"] || $session->admin["admin"] && $session->admin["group"] || $session->admin["group"] && $session->getGroupId() == $group->id)
				{
					return formatResult($group->updateIntoDB());
				}
				return formatError("You are not allowed to do this.");
			}
			return formatError("All required vars must be set.");
		}
		return formatError("Cannot select this group.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Update an Event into the database with the specified data.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Event ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function updateEvent($row)
{
	if (isUserLogged())
	{
		//$row = formatInput(zs_unserialize_array($row));
		$row = formatInput($row);
		$event = new ZS_Event();
		if ($event->selectById($row['id']))
		{
			$event->createFromArray($row);
			if ($event->checkConsistency())
			{
				$session = getUserInfos();
				if ($session->admin["super"] || $session->admin["admin"] && $session->admin["event"] || $session->admin["event"] && $session->getGroupId() == $event->id)
				{
					return formatResult($event->updateIntoDB());
				}
				return formatError("You are not allowed to do this.");
			}
			return formatError("All required vars must be set.");
		}
		return formatError("Cannot select this event.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Update an EventMember into the database with the specified data.
* Note : a session must be set in order to use this.
*
* @param	int	$id	EventMember ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function updateEventMember($row)
{
	if (isUserLogged())
	{
		//$row = formatInput(zs_unserialize_array($row));
		$row = formatInput($row);
		$eventmember = new ZS_EventMember();
		if ($eventmember->selectById($row['id']))
		{
			$eventmember->createFromArray($row);
			if ($eventmember->checkConsistency())
			{
				$session = getUserInfos();
				if ($session->admin["super"] || $session->admin["admin"] && $session->admin["eventmember"])
				{
					return formatResult($eventmember->updateIntoDB());
				}
				return formatError("You are not allowed to do this.");
			}
			return formatError("All required vars must be set.");
		}
		return formatError("Cannot select this event member.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Delete a Card from the database
* Note : a session must be set in order to use this.
*
* @param	int	$id	Card ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function deleteCard($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$card = new ZS_Card();
		if ($card->selectById($id))
		{
			// TODO
			// Delete associated eventmember ?
			$group = new ZS_Group();
			if ($group->selectById($card->group_id))
			{
				if ($card->id != $group->owner_id)
				{
					$session = getUserInfos();
					$admin = $card->getAdmin();
					if (!$admin["super"])
					{
						if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"] || $card->id == $session->id)
						{
							return formatResult($card->delete());
						}
						return formatError("You are not allowed to do this.");
					}
					return formatError("Cannot delete a super administrator account.");
				}
				return formatError("Cannot delete a group owner.");
			}
			return formatError("Cannot select this group.");
		}
		return formatError("Cannot select this card.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Delete a Group from the database
* Note : a session must be set in order to use this.
*
* @param	int	$id	Group ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function deleteGroup($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$group = new ZS_Group();
		if ($group->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["group"] || $group->owner_id == $session->id)
			{
				$cards = $group->getMembers();
				foreach ($cards as $key => $value)
				{
					$card = new ZS_Card($value);
					$card->setGroupId(0);
					$card->setAdmin(array());
				}
				return formatResult($group->delete());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this group.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Delete an Event from the database
* Note : a session must be set in order to use this.
*
* @param	int	$id	Event ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function deleteEvent($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$event = new ZS_Event();
		if ($event->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["event"] && $session->admin["eventmember"] || $session->admin["event"] && $session->admin["eventmember"] && $session->getGroupId() == $event->group_id)
			{
				$members = $event->getMembers();
				foreach ($members as $key => $value)
				{
					$member = ZS_EventMember($value);
					$member->delete();
				}
				return formatResult($event->delete());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this event.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Delete an EventMember from the database
* Note : a session must be set in order to use this.
*
* @param	int	$id	EventMember ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function deleteEventMember($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$eventmember = new ZS_EventMember();
		if ($eventmember->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["eventmember"])
			{
				return formatResult($eventmember->delete());
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this event member.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get registred server commands
*
* @return	array	All registred commands
* @access	public
*/
function commandList()
{
	global $zs_server;
	$functions = array();
	foreach ($zs_server->operations as $key => $value)
	{
		$functions[] = $key;
	}
	return formatResult($functions);
}

/**
* Test if parameters are well formated.
*
* @param	string	Serialized data
* @return	string	Parameters recognized
* @access	public
*/
function testRow($row)
{
	//$row = formatInput(zs_unserialize_array($row));
	$row = formatInput($row);
	$result = "$row:".count($row)."\n";
	if(count($row) > 0)
	{
		foreach ($row as $key => $value)
		{
			$result .= htmlspecialchars("[$key] => '$value'\n");
		}
	}
	return formatResult($result);
}

/**
* Add a Card to a Group
* Note : a session must be set in order to use this.
*
* @param	int	$card_id Card ID
* @param	int	$group_id	Group ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function addCardToGroup($card_id, $group_id)
{
	if (isUserLogged())
	{
		$card_id = formatInput($card_id);
		$group_id = formatInput($group_id);
		$card = new ZS_Card();
		$group = new ZS_Group();
		if ($card->selectById($card_id))
		{
			if ($card->group_id == 0)
			{
				if ($group->selectById($group_id))
				{
					$session = getUserInfos();
					if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"] || $session->admin["group"] && $session->admin["card"] && $session->getGroupId() == $group->id)
					{
						if ($card->setGroupId($group->id))
						{
							return formatResult(true);
						}
						return formatError("Cannot update this card.");
					}
					return formatError("You are not allowed to do this.");
				}
				return formatError("Cannot select this group.");
			}
			return formatError("This card belongs already to another group.");
		}
		return formatError("Cannot select this card.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Remove a Card from a Group
* Note : a session must be set in order to use this.
*
* @param	int	$card_id Card ID
* @param	int	$group_id	Group ID
* @return	string	TRUE if it succeeds, FALSE otherwise
* @access	public
*/
function removeCardFromGroup($card_id, $group_id)
{
	if (isUserLogged())
	{
		$card_id = formatInput($card_id);
		$group_id = formatInput($group_id);
		$card = new ZS_Card();
		$group = new ZS_Group();
		if ($card->selectById($card_id))
		{
			if ($group->selectById($group_id))
			{
				$session = getUserInfos();
				if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"] || $session->admin["group"] && $session->admin["card"] && $session->getGroupId() == $group->id)
				{
					$card->setAdmin(array());
					if ($card->setGroupId(0))
					{
						return formatResult(true);
					}
					return formatError("Cannot update this card.");
				}
				return formatError("You are not allowed to do this.");
			}
			return formatError("Cannot select this group.");
		}
		return formatError("Cannot select this card.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Get a Group list corresponding to the search term.
* The search is based on Name and on Acronym.
*
* @param	string	$search	Search string
* @return	array	Array which contains names of matching groups
* @access	public
*/
function getGroupByName($search)
{
	$search = formatInput($search);
	$group = new ZS_Group();
	return formatResult($group->searchByName($search));
}

/**
* Get a Member list corresponding to the search term.
* The search is based on username, first_name and last_name.
*
* @param	string	$search	Search string
* @return	array	Array which contains names of matching groups
* @access	public
*/
function getMemberByName($search)
{
	$search = formatInput($search);
	$card = new ZS_Card();
	return formatResult($card->searchByName($search));
}

/**
* Get a Member list corresponding to the search term which are not in a group.
* The search is based on username, first_name and last_name.
*
* @param	string	$search	Search string
* @return	array	Array which contains names of matching groups
* @access	public
*/
function getMemberByNameInGroup0($search)
{
	$search = formatInput($search);
	$card = new ZS_Card();
	return formatResult($card->searchByNameInGroup0($search));
}

/**
* Get a Member list corresponding to the search term which are in $group_id.
* The search is based on username, first_name and last_name.
*
* @param	string	$search	Search string
* @return	array	Array which contains names of matching groups
* @access	public
*/
function getMemberByNameInMyGroup($search, $group_id)
{
	$search = formatInput($search);
	$group_id = formatInput($group_id);
	$card = new ZS_Card();
	return formatResult($card->searchByNameInAgroup($search, $group_id));
}

/**
* Get a Group ID from its name.
*
* @param	string	$name	Group Name
* @return	int	Group ID
* @access	public
*/
function getGroupIdByName($name)
{
	$name = formatInput($name);
	$group = new ZS_Group();
	if ($group->selectByName($name))
	{
		return formatResult($group->id);
	}
}

/**
* Get a Group name from its ID.
*
* @param	string	$ID	Group ID
* @return	string	Group name
* @access	public
*/
function getGroupNameById($id)
{
	$id = formatInput($id);
	$group = new ZS_Group();
	if ($group->selectById($id))
	{
		return formatResult($group->name);
	}
}

/**
* Regenerate a Card Key and return it.
* Note : a session must be set in order to use this.
*
* @param	int	$id	Card ID
* @return	string	New Card Key, or FALSE.
* @access	public
*/
function regenerateCardKey($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$card = new ZS_Card();
		if ($card->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"] || $session->id == $card->id)
			$card->generateKey();
			$keystoric = new ZS_KeyStoric(array("key" => $card->key, "card_id" => $card->id));
			$keystoric->createInDB();
			if ($card->updateIntoDB())
			{
				if (isMailEnabled())
				{
					$mail = new PHPMailer(false);
					$mail->IsSendmail(); // telling the class to use SendMail transport
					$mail->AddReplyTo($session->email, "Zoku Server");
					$mail->AddAddress($card->email, $card->first_name." ".zs_strToUpper($card->last_name));
					$mail->SetFrom($session->email, "Zoku Server");
					$mail->Subject = "Zoku : Modification de la cl de votre carte";
					//$mail->AltBody = 'To view the message, please use an HTML compatible email viewer!'; // optional - MsgHTML will create an alternate automatically
					$mail->MsgHTML("<html><body><p>Bonjour,</p><p>La cl de votre carte a t rinitialise, et est maintenant :</p><p><strong>".$card->key."</strong></p><p>Veuillez prendre note que votre ancienne cl, ainsi que votre ancienne carte, sont dsormais obseltes.</p></body></html>");
					//$mail->AddAttachment('images/phpmailer.gif');      // attachment
					//$mail->AddAttachment('images/phpmailer_mini.gif'); // attachment
					$mail->Send();
				}
				return formatResult($card->key);
			}
			return formatError("Cannot update this card.");
		}
		return formatError("Cannot select this card.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Verify if a card exist, controlling with its key.
*
* @param	string	$key	Card Key
* @return	array	Public vars of the Card
* @access	public
*/
function verifyCardByKey($key)
{
	$key = formatInput($key);
	$card = new ZS_Card();
	if ($card->selectByKey($key))
	{
		return formatResult($card->getPublicVars(), true);
	}
	return formatError("Invalid key.");
}

/**
* Get all Events of session's group.
* Note : a session must be set in order to use this.
*
* @return	array	Array containing Names and ID of Events
* @access	public
*/
function getEventsOfMyGroup()
{
	if (isUserLogged())
	{
		$session = getUserInfos();
		$group = new ZS_Group();
		if ($group->selectById($session->getGroupId()))
		{
			$events = $group->getEvents();
			$result = array();
			foreach ($events as $key => $value)
			{
				$result[] = formatResult($value, true);
			}
			return formatResult($result);
		}
		return formatError("Cannot select this group.");
	}
	return formatError("A session must be set in order to use this.");
}

/**
* Register a card with its key into an event.
*
* @param	string	$key	Card Key
* @param	string	$id		Event ID
* @return	string	TRUE if succeed
* @access	public
*/
function registerCardIntoEvent($key, $id)
{
	$key = formatInput($key);
	$id = formatInput($id);
	$card = new ZS_Card();
	if ($card->selectByKey($key))
	{
		$event = new ZS_Event();
		if ($event->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["eventmember"] || $session->admin["eventmember"] && $session->getGroupId() == $event->group_id)
			{
				$data = array(	'event_id'		=>	$event->id,
								'card_id'		=>	$card->id,
								'controller_id'	=>	$session->id,
								'date'			=>	time());
				$eventmember = new ZS_EventMember($data);
				if ($result = $eventmember->createInDB())
				{
					return formatResult($result);
				}
				return formatError("Cannot create event member in database.");
			}
			return formatError("You are not allowed to do this.");
		}
		return formatError("Cannot select this event.");
	}
	return formatError("Invalid key.");
}

/**
* Set admin level of an user.
* Note : a session must be set in order to use this.
*
* @param	int		$id		Card ID
* @param	array	$admin	Admin rights, in an array
* @return	string	TRUE if succeed
* @access	public
*/
function setAdmin($id, $admin)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$admin = formatInput($admin);
		$user = new ZS_User();
		if ($user->selectById($id))
		{
			$session = getUserInfos();
			if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"] || $session->admin["group"] && $session->admin["card"] && $session->getGroupId() == $user->getGroupId())
			{
				$userAdmin = $user->getAdmin();
				$counter = 0;
				$nb_fields = 0;
				foreach ($userAdmin as $key => $value)
				{
					if ($session->admin[$key] || $session->admin[$key] == $value)
					{
						$counter++;
					}
					$nb_fields++;
				}
				if ($session->admin["super"] || $counter == $nb_fields)
				{
					$counter = 0;
					$nb_fields = 0;
					foreach ($admin as $key => $value)
					{
						if ($session->admin[$key] || $session->admin[$key] == $value)
						{
							$counter++;
						}
						$nb_fields++;
					}
					if ($session->admin["super"] || $counter == $nb_fields)
					{
						return formatResult($user->setAdmin($admin));
					}
					else
					{
						return formatError("You cannot grant a user at a greater level than yours.");
					}
				}
				else
				{
					return formatError("You cannot edit admin level of a greater account than yours.");
				}
			}
			else
			{
				return formatError("You are not allowed to do this.");
			}
		}
		else
		{
			return formatError("Cannot select this user.");
		}
	}
	else
	{
		return formatError("A session must be set in order to use this.");
	}
}

/**
* Set admin level of an user.
* Note : a session must be set in order to use this.
*
* @param	int		$id		Card ID
* @return	array	Admin rights, in an array
* @access	public
*/
function getAdmin($id)
{
	if (isUserLogged())
	{
		$id = formatInput($id);
		$user = new ZS_User();
		if ($user->selectById($id))
		{
			return formatResult($user->getAdmin());
		}
		else
		{
			return formatError("Cannot select this user.");
		}
	}
	else
	{
		return formatError("A session must be set in order to use this.");
	}
}

/**
* Get all created cards since a given date.
* Note : a session must be set in order to use this.
*
* @param	int		$date	Date, UNIX timestamp
* @return	array	Array of cards public vars
* @access	public
*/
function getLastCardsSince($date)
{
	if (isUserLogged())
	{
		$session = getUserInfos();
		if ($session->admin["super"] || $session->admin["admin"] && $session->admin["card"])
		{
			$date = formatInput($date);
			$card = new ZS_Card();
			return formatResult($card->getLastSince($date));
		}
		else
		{
			return formatError("You are not allowed to do this.");
		}
	}
	else
	{
		return formatError("A session must be set in order to use this.");
	}
}

/**
* Get all created groups since a given date.
* Note : a session must be set in order to use this.
*
* @param	int		$date	Date, UNIX timestamp
* @return	array	Array of groups public vars
* @access	public
*/
function getLastGroupsSince($date)
{
	if (isUserLogged())
	{
		$session = getUserInfos();
		if ($session->admin["super"] || $session->admin["admin"] && $session->admin["group"])
		{
			$date = formatInput($date);
			$group = new ZS_Group();
			return formatResult($group->getLastSince($date));
		}
		else
		{
			return formatError("You are not allowed to do this.");
		}
	}
	else
	{
		return formatError("A session must be set in order to use this.");
	}
}

/**
* Get all created events since a given date.
* Note : a session must be set in order to use this.
*
* @param	int		$date	Date, UNIX timestamp
* @return	array	Array of events public vars
* @access	public
*/
function getLastEventsSince($date)
{
	if (isUserLogged())
	{
		$session = getUserInfos();
		if ($session->admin["super"] || $session->admin["admin"] && $session->admin["event"])
		{
			$date = formatInput($date);
			$event = new ZS_Event();
			return formatResult($event->getLastSince($date));
		}
		else
		{
			return formatError("You are not allowed to do this.");
		}
	}
	else
	{
		return formatError("A session must be set in order to use this.");
	}
}

?>